import phonenumbers
import requests
from phonenumbers import carrier
from requests.adapters import HTTPAdapter
from requests.packages.urllib3.util.retry import Retry

from .errors import ValidationError


def requests_retry_session(
    retries=3, backoff_factor=0.3, status_forcelist=(502, 504), session=None, **kwargs
):
    session = session or requests.Session()
    retry = Retry(
        total=retries,
        read=retries,
        connect=retries,
        backoff_factor=backoff_factor,
        status_forcelist=status_forcelist,
    )
    adapter = HTTPAdapter(max_retries=retry)
    session.mount("http://", adapter)
    session.mount("https://", adapter)
    return session


def validate_phone_number(number):
    phone_obj = phonenumbers.parse(number, "UG")
    if not phonenumbers.is_valid_number(phone_obj):
        raise ValidationError(f"Invalid phone number {number}")
    if carrier.name_for_number(phone_obj, "en") not in ["MTN", "Airtel"]:
        raise ValidationError(
            f"{number}: Only MTN and Airtel are supported at the moment."
        )

    return f"256{phone_obj.national_number}"


def validate_string(_string):
    if not isinstance(_string, str):
        raise ValidationError(f"{_string}: Must be a string.")

    return _string


def validate_number(number):
    number_types = (int, float)
    if not type(number) in number_types:
        raise ValidationError(f"{number}: Must be a number")

    return number



from .client import ZengaPayAPI
from .utils import validate_number, validate_phone_number, validate_string


class Collections(ZengaPayAPI):
    def collect(self, payload):
        payload["msisdn"] = validate_phone_number(payload.get("msisdn"))
        payload["amount"] = validate_number(payload.get("amount"))
        payload["external_reference"] = validate_string(
            payload.get("external_reference")
        )
        payload["narration"] = validate_string(payload.get("narration"))

        url = f"{self.config.base_url}/collections"
        res = self.request("POST", url, post_data=payload)

        return res.json()

    def get_all_collections(self, url=None):
        if url is None:
            url = f"{self.config.base_url}/collections"
        res = self.request("GET", url)

        return res.json()

    def get_collection(self, transaction_ref):
        url = f"{self.config.base_url}/collections/{validate_string(transaction_ref)}"
        res = self.request("GET", url)

        return res.json()

    def get_transaction_status(self, transaction_ref):
        url = f"{self.config.base_url}/collections/{validate_string(transaction_ref)}"
        res = self.request("GET", url)

        return res.json()["data"]["transactionStatus"]



# zengapay/__init__.py

# version of the zengapay package
__version__ = "0.2.0"


from .accounts import Accounts  # noqa: F401
from .collection import Collections  # noqa: F401
from .transfer import Transfers  # noqa: F401



from .client import ZengaPayAPI
from .utils import validate_number, validate_phone_number, validate_string


class Accounts(ZengaPayAPI):
    def get_account_balance(self):
        url = f"{self.config.base_url}/account/balance"
        res = self.request("GET", url)

        return res.json()

    def get_account_statement(self, url=None, limit=0, **kwargs):
        designations = ["TRANSACTION", "CHARGES"]
        currency_codes = ["UGX-MTNMM", "UGX-WARIDMM"]
        status = ["FAILED", "PENDING", "INDETERMINATE", "SUCCEEDED"]
        if url is None:
            url = f"{self.config.base_url}/account/statement?limit={validate_number(limit)}"
        if (
            "designation" in kwargs
            and kwargs.get("designation").upper() in designations
        ):
            url += f"&designation={kwargs.get('designation')}"
        if (
            "currency_code" in kwargs
            and kwargs.get("currency_code").upper() in currency_codes
        ):
            url += f"&currency_code={kwargs.get('currency_code')}"
        if "status" in kwargs and kwargs.get("status").upper() in status:
            url += f"&status={kwargs.get('status')}"
        if "start" in kwargs:
            url += f"&start={kwargs.get('start')}"
        if "end" in kwargs:
            url += f"&end={kwargs.get('end')}"
        res = self.request("GET", url)

        return res.json()



"""
Base Implementation of the ZengaPay API client

@author: Patrick Walukagga
"""


import json

import requests
from requests import Request, Session
from requests.auth import AuthBase

from .config import ZengaPayConfig
from .utils import requests_retry_session

try:
    from json.decoder import JSONDecodeError
except ImportError:
    JSONDecodeError = ValueError


class Response:
    def __init__(self, body, code, headers):
        self.body = body
        self.code = code
        self.headers = headers
        self.data = body


class ZengaPayAuth(AuthBase):
    """Attaches Authentation to the given Requesst Object."""

    def __init__(self, token):
        self.token = token

    def __call__(self, r):
        """Attach an API user token to a custom auth header."""

        # Modify and return the request
        r.headers["Authorization"] = f"Bearer {self.token}"
        return r


class ClientInterface:
    def get_auth_token(self):
        raise NotImplementedError

    def get_transaction_status(self):
        return NotImplementedError


class Client(ClientInterface):
    def get_auth_token(self):
        return super(Client, self).get_auth_token()

    def get_transaction_status(self):
        return super(Client, self).get_transaction_status()


class ZengaPayAPI(ClientInterface):
    def __init__(self, config, **kwargs):
        super(ZengaPayAPI, self).__init__(**kwargs)
        self._session = Session()
        self._config = ZengaPayConfig(config)

    @property
    def config(self):
        return self._config

    def request(self, method, url, post_data=None):
        self.auth_token = self.get_auth_token()
        request = Request(
            method, url, data=post_data, auth=ZengaPayAuth(self.auth_token)
        )

        prep_req = self._session.prepare_request(request)
        resp = requests_retry_session(session=self._session).send(
            prep_req, verify=False
        )

        return resp

    def get_auth_token(self):
        return self.config.api_token



import pathlib
from setuptools import find_packages, setup

# The directory containing this file
HERE = pathlib.Path(__file__).parent

# The text of the README file
README = (HERE / "README.md").read_text()

# This call to setup() does all the work
setup(
    name="zengapay",
    version="0.2.0",
    description="Python Wrapper for the ZengaPay API",
    long_description=README,
    long_description_content_type="text/markdown",
    url="https://github.com/zengapay/zengapay-python",
    author="Patrick Walukagga",
    author_email="pwalukagga@gmail.com",
    python_requires=">=3.7.0",
    license="MIT",
    classifiers=[
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
    ],
    keywords=["ZengaPay", "ZengaPay API", "ZengaPay Python Wrapper", "ZengaPay Python"],
    packages=find_packages(exclude=["tests", "*.tests", "*.tests.*", "tests.*"]),
    include_package_data=True,
    install_requires=["requests", "phonenumbers"],
    # entry_points={
    #     "console_scripts": [
    #     ]
    # },
)



from .errors import ConfigurationError


class ZengaPayConfig(object):
    def __init__(self, config):
        """
        config = {
            ZENGAPAY_ENVIRONMENT: os.environ.get("ZENGAPAY_APP_SETTINGS", "sandbox"),
            ZENGAPAY_BASE_URL: os.environ.get("ZENGAPAY_BASE_URL", "https://api.sandbox.zengapay.com/v1/"),
            ZENGAPAY_USER_API_TOKEN: os.environ.get("ZENGAPAY_USER_API_TOKEN")
        }
        """

        self.config = config

    @property
    def environment(self):
        env = self.config["ZENGAPAY_ENVIRONMENT"]
        if not env:
            raise ConfigurationError(
                "ZENGAPAY_ENVIRONMENT is missing in configuration."
            )
        return env

    @property
    def base_url(self):
        base_url = self.config["ZENGAPAY_BASE_URL"]
        if not base_url:
            raise ConfigurationError("ZENGAPAY BASE URL is missing in configuration.")
        return base_url

    @property
    def api_token(self):
        token = self.config["ZENGAPAY_USER_API_TOKEN"]
        if not token:
            raise ConfigurationError(
                "ZENGAPAY_USER_API_TOKEN is missing in configuration."
            )
        return token
